open Core.Std open OUnit open Block

let raises_exception ?(exn) f =
  match exn with
  |Some e ->
    (try f (); false
    with e' when e=e' -> true)
  |None   ->
    (try f (); false
    with _ -> true)

module Values =
  struct
    include Schema_test.Values
    let new_bck schs = Block.of_schs_and_cstr ~cstr:Constraint.top ~schs
    let bck_p1 = new_bck [p1]
    let bck_pn = new_bck [p_n]
    let bck_npn = new_bck [np_n]
    let bck_pnminus1 = new_bck [p_nminus1]
    let bck_npnminus1 = new_bck [np_nminus1]
    let bck_pn_pnp1 = new_bck [p_n;p_nplus1]
    let bck_pnp1_pn = new_bck [p_nplus1;p_n]
    let bck_pn_pnm1 = new_bck [p_n;p_nminus1]
    let bck_pnplus1 = new_bck [p_nplus1]
    let bck_p1_pnp1_pn = new_bck [p1;p_nplus1;p_n]
    let bck_p1_pn_pnm1 = new_bck [p1;p_n;p_nminus1]

    let bck_itand1 = new_bck [itand1]
    let bck_itand1' = new_bck [itand1']
  end

open Values

let test =
  "block" >:::
    [
      "equal" >::
        (fun () ->
          "[p1] [p1]"                 @? (equal bck_p1 bck_p1);
          "[pn] [pn]"                 @? (equal bck_pn bck_pn);
          "[~pn] [~pn]"               @? (equal bck_npn bck_npn);
          "[pn-1] [pn-1]"             @? (equal bck_pnminus1 bck_pnminus1);
          "[pn;pn+1] [pn;pn+1]"       @? (equal bck_pn_pnp1 bck_pn_pnp1);
          "[pn;pn-1] [pn;pn-1]"       @? (equal bck_pn_pnm1 bck_pn_pnm1);
          "[p1;pn+1;pn] [p1;pn+1;pn]" @? (equal bck_p1_pnp1_pn bck_p1_pnp1_pn);
          "[p1] [pn]"                 @? (not (equal bck_p1 bck_pn));
          "[pn] [p1]"                 @? (not (equal bck_pn bck_p1));
        );
      "normalize" >::
        (fun () ->
          "[p1]" @? (normalize bck_p1 = bck_p1);
          "[pn]" @? (normalize bck_pn = bck_pn);
          "[pn+1]" @? (normalize bck_pnplus1 = bck_pn);
          "[pn;pn+1]" @? (normalize bck_pn_pnp1 = bck_pn_pnp1);
          "[pn+1;pn]" @? (normalize bck_pnp1_pn = bck_pn_pnm1);
          "[p1;pn+1;pn]" @? (normalize bck_p1_pnp1_pn = bck_p1_pn_pnm1);
          "[itand1]" @? (normalize bck_itand1 = bck_itand1);
          "[itand1']" @? (normalize bck_itand1' = bck_itand1);
        );
      "looping map" >:::
        let empty_set = LoopingMap.empty in
        let add set bck = LoopingMap.add set ~key:bck ~data:() in
        let p1_set = add       empty_set bck_p1 in
        let pn_set = add       empty_set bck_pn in
        let npn_set = add      empty_set bck_npn in
        let pnminus1_set = add empty_set bck_pnminus1 in
        let itand1_set = add   empty_set bck_itand1 in
        (*let itand1'_set = add empty_set bck_itand1' in*)
        (*let some_set1 = add (add p1_set np_nplus1) itand8 in*)
        [
          "loops_among" >::
            (fun () ->
              "[P1] -> [P1]"                                   @? (Option.is_some (LoopingMap.find p1_set ~key:bck_p1));
              "[Pn] -> [Pn]"                                   @? (Option.is_some (LoopingMap.find pn_set ~key:bck_pn));
              "[Pn-1] -> [Pn-1]"                               @? (Option.is_some (LoopingMap.find pnminus1_set ~key:bck_pnminus1));
              "[Pn-1] -> [Pn]"                                 @? (Option.is_some (LoopingMap.find pn_set ~key:bck_pnminus1));
              "[~Pn-1] -> [~Pn]"                               @? (Option.is_some (LoopingMap.find npn_set ~key:bck_npnminus1));
              "/\\i=1..n P_i -> P_i+1 -> /\\i=1..n ..."        @? (Option.is_some (LoopingMap.find itand1_set ~key:bck_itand1));
              "/\\i=1..n-1 P_i -> P_i+1 -> /\\i=1..n ..."      @? (Option.is_some (LoopingMap.find itand1_set ~key:bck_itand1'));
              (*"/\\i=1..n P_i -> P_i+1 -> /\\i=1..n ..."      @? (Option.is_some (LoopingMap.find itand8_set itand8));*)
              (*"P_1 -> {P_1;...}"                             @? (Option.is_some (LoopingMap.find some_set1 p1));*)
              (*"~P_n+1 -> {~P_n+1;...}"                       @? (Option.is_some (LoopingMap.find some_set1 np_nplus1));*)
              (*"/\\i=1..n-1 P_i -> P_i+1 -> {/\\i=1..n-1;..}" @? (Option.is_some (LoopingMap.find some_set1 itand8));*)
            );
        ]
    ]

